@@ -1,6 +1,9 @@
 # encoding: utf-8
+
 require 'cri/version'
 
+# The namespace for Cri, a library for building easy-to-use commandline tools
+# with support for nested commands.
 module Cri
 
   # A generic error class for all Cri-specific errors.
@@ -18,12 +21,16 @@ module Cri
   end
 
   autoload 'Command',           'cri/command'
+  autoload 'StringFormatter',   'cri/string_formatter'
   autoload 'CommandDSL',        'cri/command_dsl'
   autoload 'CommandRunner',     'cri/command_runner'
+  autoload 'HelpRenderer',      'cri/help_renderer'
   autoload 'OptionParser',      'cri/option_parser'
+  autoload 'Platform',          'cri/platform'
 
 end
 
 require 'set'
 
 require 'cri/core_ext'
+require 'cri/argument_array'
